==> mhelper-1.0.1.46.tar.gz: mhelper-1.0.1.46/mhelper/special_types.py <==
"""
Special string types
"""
from enum import Enum
from typing import TypeVar

from flags import Flags


from mhelper.generics_helper import MAnnotationFactory, MAnnotation


T = TypeVar( "T" )


class NoDefaultType:
    """
    Type of `NOT_PROVIDED` constant. Used to distinguish a value of `None` from a value that isn't provided.
    """
    
    
    def __str__( self ) -> str:
        return "(not_provided)"


NOT_PROVIDED = NoDefaultType()
"""
NOT_PROVIDED is used to distinguish between a value of `None` and a value that that isn't even provided.

As a parameter default, it can be considered to mean "mandatory".
When passed as a parameter's value, it can be considered to mean "I'm not specifying this, use your default (or raise an Exception if there is no default)."
"""


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ ENUMS ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒



class MEnum( Enum ):
    """
    An enum class that presents a less useless string function
    """
    
    
    def __str__( self ):
        return self.name


class MFlags( Flags ):
    """
    A flags class that presents  less useless string function
    """
    __no_flags_name__ = "NONE"
    __all_flags_name__ = "ALL"
    
    
    def __str__( self ):
        return self.to_simple_str()


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ FILE NAME ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒



class EFileMode( MEnum ):
    """
    Used by `FileNameAnnotation`.
    How file is written to.
    
    :data UNKNOWN:  Default
    :data READ:     Or `"r"`, open file for reading
    :data WRITE:    Or `"w"`, open file for writing
    :data OUTPUT:   Or `"o"`, open file for writing, accepts output names supported by :func:`io_helper.open_write`.
    """
    UNKNOWN = 0
    READ = 1
    WRITE = 2
    OUTPUT = 3


_default_filename_path = None


class FileNameAnnotation( MAnnotation ):
    def __init__( self, args ):
        super().__init__( args )
        
        mode = EFileMode.UNKNOWN
        ext = None
        
        for p in self.parameters:
            if isinstance( p, EFileMode ):
                mode = p
            elif p == "r":
                mode = EFileMode.READ
            elif p == "w":
                mode = EFileMode.WRITE
            elif isinstance( p, str ):
                ext = p
            else:
                from mhelper.exception_helper import SwitchError
                raise SwitchError( "parameter", p, instance = True )
        
        self.mode = mode
        self.extension = ext
    
    
    def __str__( self ):
        assert self.child is str
        return "{}[{} {}]".format( self.factory, self.mode.name, self.extension )


Filename: FileNameAnnotation = MAnnotationFactory( "Filename", annotation_type = FileNameAnnotation )[str]
"""File names"""

Dirname = MAnnotationFactory( "Dirname" )[str]
"""Directory names"""

MOptional = MAnnotationFactory( "MOptional" )
"""MOptional is Optional, but doesn't complain about the argument not being a type.
It will be redundant when typing"""

MUnion = MAnnotationFactory( "MOptional" )
"""MOptional is Optional, but doesn't complain about the argument not being a type.
It will be redundant when typing"""


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ PASSWORD  ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒



class Password:
    """
    Passwords
    """
    
    
    def __init__( self, value: str ):
        self.__value = value
    
    
    @property
    def value( self ):
        return self.__value
    
    
    def __str__( self ) -> str:
        return "********"


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ READ-ONLY ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒


HReadonly = MAnnotationFactory( "Readonly" )

==> mhelper-1.0.1.52.tar.gz: mhelper-1.0.1.52/mhelper/special_types.py <==
"""
Special string types
"""
from enum import Enum
from typing import TypeVar, Optional

# noinspection PyPackageRequirements
from flags import Flags

from mhelper.generics_helper import MAnnotationFactory, MAnnotation


T = TypeVar( "T" )


class Sentinel:
    """
    Type used for sentinel objects (things that don't do anything but whose presence indicates something).
    The Sentinel also has a `str` method equal to its name, so is appropriate for user feedback. 
    """
    
    
    def __init__( self, name: str ):
        """
        :param name:    Name, for debugging or display. 
        """
        self.__name = name
    
    
    def __str__( self ) -> str:
        return self.__name
    
    
    def __repr__( self ):
        return "Sentinel({})".format( repr( self.__name ) )


NOT_PROVIDED = Sentinel( "(Not provided)" )
"""
NOT_PROVIDED is used to distinguish between a value of `None` and a value that that isn't even provided.
"""


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ ENUMS ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒


class MEnum( Enum ):
    """
    An enum class that presents a less useless string function
    """
    
    
    def __str__( self ):
        return self.name


class MFlags( Flags ):
    """
    A flags class that presents  less useless string function
    """
    __no_flags_name__ = "NONE"
    __all_flags_name__ = "ALL"
    
    
    def __str__( self ):
        return self.to_simple_str()


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ FILE NAME ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒


class EFileMode( MEnum ):
    """
    Used by `FileNameAnnotation`.
    How file is written to.
    
    :cvar UNKNOWN:  Default
    :cvar READ:     Or `"r"`, open file for reading
    :cvar WRITE:    Or `"w"`, open file for writing
    :cvar OUTPUT:   Or `"o"`, open file for writing, accepts output names supported by :func:`io_helper.open_write`.
    """
    UNKNOWN = 0
    READ = 1
    WRITE = 2
    OUTPUT = 3


_default_filename_path = None


class FileNameAnnotation( MAnnotation ):
    def __init__( self, args ):
        super().__init__( args )
        
        mode = EFileMode.UNKNOWN
        ext = None
        
        for p in self.parameters:
            if isinstance( p, EFileMode ):
                mode = p
            elif p == "r":
                mode = EFileMode.READ
            elif p == "w":
                mode = EFileMode.WRITE
            elif isinstance( p, str ):
                ext = p
            else:
                from mhelper.exception_helper import SwitchError
                raise SwitchError( "parameter", p, instance = True )
        
        self.mode = mode
        self.extension = ext
    
    
    def __str__( self ):
        assert self.child is str
        return "{}[{} {}]".format( self.factory, self.mode.name, self.extension )


class EnumerativeAnnotation( MAnnotation ):
    def __init__( self, args ):
        super().__init__( args )
        
        opts = { }
        
        for p in self.parameters:
            if isinstance( p, dict ):
                opts.update( p )
        
        self.options = opts


Filename: FileNameAnnotation = MAnnotationFactory( "Filename", annotation_type = FileNameAnnotation )[str]
"""File names"""

NamedBoolean = MAnnotationFactory( "Boolean", annotation_type = EnumerativeAnnotation )[bool]

NamedTristate = MAnnotationFactory( "Tristate", annotation_type = EnumerativeAnnotation )[Optional[bool]]

Dirname = MAnnotationFactory( "Dirname" )[str]
"""Directory names"""

MOptional = MAnnotationFactory( "MOptional" )
"""MOptional is Optional, but doesn't complain about the argument not being a type.
It will be redundant when typing"""

MUnion = MAnnotationFactory( "MOptional" )
"""MOptional is Optional, but doesn't complain about the argument not being a type.
It will be redundant when typing"""


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ PASSWORD  ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒


class Password:
    """
    Passwords
    """
    
    
    def __init__( self, value: str ):
        self.__value = value
    
    
    @property
    def value( self ):
        return self.__value
    
    
    def __str__( self ) -> str:
        return "********"


# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒ READ-ONLY ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
# ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒


HReadonly = MAnnotationFactory( "Readonly" )
